import java.io.*;
import java.net.*;
import java.util.*;
public class ServidorDeChat extends Thread 
{
	public static void main(String args[]) 
	{
		// instancia o vetor de clientes conectados
		clientes = new Vector();
        
		try 
		{
			// criando um socket que fica escutando a porta 2222.
			ServerSocket s = new ServerSocket(2222);
           
			// loop principal.
			while (true) 
			{
				// aguarda algum se conectar. A execuo do servidor fica bloqueada na 
				// chamada do mtodo accept da classe ServerSocket. Quando algum se 
				// conectar ao servidor, o mtodo desbloqueia e retorna com um objeto da 
				// classe Socket, que  uma ponta da comunicao.
				System.out.print("Esperando alguem se conectar...");
				Socket conexao = s.accept();
				System.out.println(" Conectou!");
                
				// cria uma nova thread para tratar essa conexo
				Thread t = new ServidorDeChat(conexao);
				t.start();
                
				// voltando ao loop, esperando mais algum se conectar.
			}
		} 
		catch (IOException e) 
		{
			// caso ocorra alguma exceo de I/O, mostre qual foi.
			System.out.println("IOException: " + e);
		}
	}
    
	// 
	// parte da classe que controla as conexes atravs de threads
	// 
    
	// vetor de clientes conectados (ateno ao static, pois se quer uma instncia do vetor)
	private static Vector clientes;
	// socket deste cliente
	private Socket conexao; 
	// nome deste cliente
	private String meuNome; 
    
	// construtor que recebe o socket deste cliente
	public ServidorDeChat(Socket s) 
	{
		conexao = s;
	}
    
	// execuo da thread
	public void run() 
	{
		try 
		{
			// obtendo os objetos que permitem controlar o fluxo de comunicao
			BufferedReader entrada = 
				new BufferedReader(new InputStreamReader(conexao.getInputStream()));
			PrintStream saida = new PrintStream(conexao.getOutputStream());

			// primeiramente, espera-se pelo nome do cliente
			meuNome = entrada.readLine();
            
			// agora, deve-se analisar se o retorno do servidor  uma string vlida, pois 
			// pode ser que a conexo tenha sido interrompida e, nesse caso, a string  null. 
			// Se isso ocorrer, deve-se terminar a execuo.
			if (meuNome == null) 
			{
				return;
			}
            
			// uma vez que se tem um cliente conectado e conhecido (nome), coloca-se 
			// o fluxo de sada para este cliente no vetor de clientes conectados
			clientes.add(saida);
			// obs: note que clientes  um objeto compartilhado por vrias threads! 
			// De acordo com o manual da API, os mtodos so sincronizados e, 
			// portanto, no se deve preocupar com os acessos simultneos.
			// loop principal: esperando por alguma string do cliente, para enviar de 
			// volta para todos os clientes conectados, at que ele envie uma linha 
			// em branco. Deve-se primeiro verificar se a linha recebida no  nula (null). 
			// isso pode ocorrer quando a conexo  interrompida pelo cliente. Se a 
			// linha no for null (o objeto existe) pode-se utilizar os mtodos de 
			// comparao de string (caso contrrio, estaria tentando chamar um mtodo 
			// de um objeto que no existe).
			String linha = entrada.readLine();
			while (linha != null && !(linha.trim().equals(""))) 
			{
				// envia a linha de volta para todos os clientes conectados
				sendToAll(saida, " disse: ", linha);
				// espera por uma nova linha.
				linha = entrada.readLine();
			}

			// uma vez que o cliente enviou uma linha em branco, pode-se retirar o fluxo 
			// de sada do vetor de clientes e depois fechar a conexo
			sendToAll(saida, " saiu ", "do chat!");
			clientes.remove(saida);
			conexao.close();
		}
		catch (IOException e)
		{
			// caso ocorra alguma exceo de I/O, mostre qual foi.
			System.out.println("IOException: " + e);
		}
	}
    
	// enviar uma mensagem para todos os usurios, menos para o prprio
	public void sendToAll(PrintStream saida, String acao, String linha) throws IOException 
	{
		Enumeration e = clientes.elements();
		while (e.hasMoreElements()) 
		{
			// obtm o fluxo de sada de um dos clientes
			PrintStream chat = (PrintStream) e.nextElement();

			// envia para todos, menos para o prprio usurio
			if (chat != saida) 
			{
				chat.println(meuNome + acao + linha);
			}
		}
	}
}